/*
*
*  UserGroupsAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

public class UserGroupsAction extends BaseAction {

  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;

  // Load default values into keyword hashtable
  static {
    default_keywords = new RMHashtable();
    RMProperty gp = new RMProperty("NAME","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("GROUPS","");
    default_keywords.put(gp.name, gp);
  }

  public UserGroupsAction(Repository repository, int action, StreamTokenizer st,
         int lineno) throws IOException{
    super(action, st, lineno);
    this.repository = repository;
    this.action = action;
    this.type = "USERGROUPS";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  public int doAction() {
    switch (action) {
    case RMLoader.ADD :
      addUserGroups();
      break;
    case RMLoader.REMOVE :
      removeUserGroups();
      break;
    case RMLoader.SET :
      setUserGroups();
    }
    return 0;
  }

  public void addUserGroups() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    User user;

    try {
      user = repository.getUser(name);
    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Couldn't find user \"" + rmp.value + "\".");
      return;
    }

    try {
      rmp = (RMProperty)keywords.get("GROUPS");
      String groups[] = parseCommaSeparatedList(rmp.value);
      Vector add_groups = new Vector();
      for (int i = 0; i < groups.length; i++)
        add_groups.addElement(repository.getGroup(groups[i]));
      QueryVector qv = user.getGroups();
      Vector v = new Vector();
      for (int i = 0; i < qv.size(); i++) 
        v.addElement(((Group)qv.elementAt(i)));
      for (int i = 0; i < add_groups.size(); i++)
        if (!v.contains(add_groups.elementAt(i))) 
          user.addGroup((Group)add_groups.elementAt(i));
      user.update();
      RMLoader.logMsg("Successfully added user " + name + " to groups " + 
                      rmp.value);
    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error adding user " + name + " to groups " + rmp.value);
    }
  } 

  private void removeUserGroups() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      User user = repository.getUser(name);
      rmp = (RMProperty)keywords.get("GROUPS");
      String groups[] = parseCommaSeparatedList(rmp.value);
      Vector rem_groups = new Vector();
      for (int i = 0; i < groups.length; i++)
        user.removeGroup(repository.getGroup(groups[i]));
      user.update();
      RMLoader.logMsg("Successfully removed user " + name + " from groups " + 
                      rmp.value);
    }
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error removing user " + name + " from groups " +
                      rmp.value);
    }
  }

  private void setUserGroups() {
    String groups[] = null;
    User user = null;

    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      user = repository.getUser(name);

      // First, we remove the user from all current groups 
      // (except the default group)
      QueryVector qv = user.getGroups();
      Group defaultGroup = user.getDefaultGroup();
      Group g;
      for (int i = 0; i < qv.size(); i++) {
        g = (Group)qv.elementAt(i);
        if ( ! g.equals(defaultGroup) ) 
           user.removeGroup(g);
        }
      //System.out.println("Done removing.");

      // Then we add the user into the groups specified.
      rmp = (RMProperty)keywords.get("GROUPS");
      groups = parseCommaSeparatedList(rmp.value);
    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error setting groups for user " + name + ": " +
                       rme.getMessage());
      return;
    }
     
    try {
      for (int i = 0; i < groups.length; i++)  
        user.addGroup(repository.getGroup(groups[i]));
      user.update();
      RMLoader.logMsg("Successfully set user " + name + " as groups " + 
                      rmp.value);
    }
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error setting user " + name + " as groups " +
                      rmp.value + ": " + rme.getMessage());
    }
  }
}
